@@ -3,8 +3,8 @@ Tests for the conf submodule.
 """
 
 import numpy as np
-from vip_hci.conf.mem import check_enough_memory
-from vip_hci.conf.utils_conf import check_array
+from vip_hci.config.mem import check_enough_memory
+from vip_hci.config.utils_conf import check_array
 
 
 def test_check_enough_memory():
