@@ -2,7 +2,7 @@
 Subpackage ``metrics`` includes:
     - signal-to-noise (S/N) estimation,
     - S/N map generation,
-    - STIMP map generation,
+    - STIM map generation,
     - detection of point like sources (for pipelines),
     - fake companions injection,
     - fake disks generation and injection,
@@ -12,9 +12,6 @@ Subpackage ``metrics`` includes:
 """
 from .contrcurve import *
 from .detection import *
-from .fakecomp import *
 from .roc import *
 from .snr_source import *
-from .fakedisk import *
-from .scattered_light_disk import *
 from .stim import *
