@@ -39,9 +39,9 @@ else:
 from multiprocessing import cpu_count
 from matplotlib import pyplot as plt
 from . import frame_crop
-from ..conf import time_ini, timing, Progressbar
-from ..conf.utils_conf import vip_figsize, check_array
-from ..conf.utils_conf import pool_map, iterable
+from ..config import time_ini, timing, Progressbar
+from ..config.utils_conf import vip_figsize, check_array
+from ..config.utils_conf import pool_map, iterable
 from ..stats import frame_basic_stats
 from ..var import (get_square, frame_center, get_annulus_segments,
                    fit_2dmoffat, fit_2dgaussian, fit_2dairydisk,
@@ -78,13 +78,16 @@ def frame_shift(array, shift_y, shift_x, imlib='vip-fft',
         'lanczos4' the slowest and accurate. 'lanczos4' is the default for
         Opencv and 'biquartic' for Scipy.ndimage.
     border_mode : {'reflect', 'nearest', 'constant', 'mirror', 'wrap'}
-        Points outside the boundaries of the input are filled accordingly.
+        For 'opencv' and 'ndimage-interp', points outside the boundaries of the 
+        input are filled according to tge value of this parameter.
         With 'reflect', the input is extended by reflecting about the edge of
         the last pixel. With 'nearest', the input is extended by replicating the
         last pixel. With 'constant', the input is extended by filling all values
         beyond the edge with zeros. With 'mirror', the input is extended by
         reflecting about the center of the last pixel. With 'wrap', the input is
         extended by wrapping around to the opposite edge. Default is 'reflect'.
+        Note: for 'ndimage-fourier' default is 'wrap' (impossible to change), 
+        while border_mode is 'constant' (zeros) for 'vip-fft'.
 
     Returns
     -------
@@ -95,11 +98,81 @@ def frame_shift(array, shift_y, shift_x, imlib='vip-fft',
     check_array(array, dim=2)
     image = array.copy()
 
-    if imlib == 'ndimage-fourier' or imlib == 'vip-fft':
+    if imlib == 'ndimage-fourier':
+        # Warning: default border mode is 'wrap' (cannot be changed)
         shift_val = (shift_y, shift_x)
         array_shifted = fourier_shift(np.fft.fftn(image), shift_val)
         array_shifted = np.fft.ifftn(array_shifted)
         array_shifted = array_shifted.real
+        
+    elif imlib == 'vip-fft':
+        ny_ori, nx_ori = image.shape
+            
+        # First pad to avoid 'wrapping' values at the edges
+        npad = int(np.ceil(np.amax(np.abs([shift_y,shift_x]))))
+        cy_ori, cx_ori = frame_center(array)
+        new_y = int(ny_ori+2*npad)
+        new_x = int(nx_ori+2*npad)
+        new_image = np.zeros([new_y,new_x], dtype=array.dtype)
+        cy, cx = frame_center(new_image)
+        y0 = int(cy-cy_ori)
+        y1 = int(cy+cy_ori)
+        if new_y%2:
+            y1+=1
+        x0 = int(cx-cx_ori)
+        x1 = int(cx+cx_ori)
+        if new_x%2:
+            x1+=1
+        new_image[y0:y1,x0:x1] = array.copy()
+        p_y0 = npad
+        p_x0 = npad
+        npix = new_y
+
+        # If non-square, add extra pad to make it square
+        if new_y != new_x:
+            if new_y > new_x:
+                npix = new_y
+                image = np.zeros([npix, npix])
+                x0 = int(cy-cx)
+                x1 = x0+new_x
+                image[:,x0:x1] = new_image.copy()
+                p_x0 += x0
+            else:
+                npix = new_x
+                image = np.zeros([npix, npix])
+                y0 = int(cx-cy)
+                y1 = y0+new_y
+                image[y0:y1] = new_image.copy()
+                p_y0 += y0
+            new_image = image.copy()
+                
+        # If odd, add an extra pad layer to make it even
+        if npix%2:
+            npix+=1
+            image = np.zeros([npix,npix])
+            if shift_x>0:
+                x0=0
+            else:
+                x0=1
+                p_x0+=1
+            if shift_y>0:
+                y0=0
+            else:
+                y0=1
+                p_y0+=1
+            image[y0:y0+npix-1,x0:x0+npix-1] = new_image.copy()
+            new_image = image.copy()
+    
+        # actual FT-based shift
+        ramp = np.outer(np.ones(npix), np.arange(npix) - npix/2)
+        tilt = (-2*np.pi / npix) * (shift_x*ramp + shift_y*ramp.T)
+        fact = np.fft.fftshift(np.cos(tilt) + 1j*np.sin(tilt))
+    
+        image_ft = np.fft.fft2(new_image)  # no np.fft.fftshift applied!
+        array_shifted = np.fft.ifft2(image_ft * fact).real
+        
+        # final crop to compensate padding
+        array_shifted = array_shifted[p_y0:p_y0+ny_ori, p_x0:p_x0+nx_ori]
 
     elif imlib == 'ndimage-interp':
         if interpolation == 'nearneig':
@@ -168,7 +241,7 @@ def frame_shift(array, shift_y, shift_x, imlib='vip-fft',
 
 
 def cube_shift(cube, shift_y, shift_x, imlib='vip-fft', 
-               interpolation='lanczos4'):
+               interpolation='lanczos4', border_mode='reflect'):
     """ Shifts the X-Y coordinates of a cube or 3D array by x and y values.
 
     Parameters
@@ -182,7 +255,9 @@ def cube_shift(cube, shift_y, shift_x, imlib='vip-fft',
         See the documentation of the ``vip_hci.preproc.frame_shift`` function.
     interpolation : str, optional
         See the documentation of the ``vip_hci.preproc.frame_shift`` function.
-
+    border_mode : str, optional
+        See the documentation of the ``vip_hci.preproc.frame_shift`` function.
+        
     Returns
     -------
     cube_out : numpy ndarray, 3d
@@ -200,13 +275,14 @@ def cube_shift(cube, shift_y, shift_x, imlib='vip-fft',
 
     for i in range(cube.shape[0]):
         cube_out[i] = frame_shift(cube[i], shift_y[i], shift_x[i], imlib,
-                                  interpolation)
+                                  interpolation, border_mode)
     return cube_out
 
 
 def frame_center_satspots(array, xy, subi_size=19, sigfactor=6, shift=False,
-                          imlib='vip-fft', interpolation='lanczos4',
-                          fit_type='moff', debug=False, verbose=True):
+                          imlib='vip-fft', interpolation='lanczos4', 
+                          fit_type='moff', border_mode='reflect', debug=False, 
+                          verbose=True):
     """ Finds the center of a frame with waffle/satellite spots (e.g. for
     VLT/SPHERE). The method used to determine the center is by centroiding the
     4 spots via a 2d Gaussian fit and finding the intersection of the
@@ -240,6 +316,14 @@ def frame_center_satspots(array, xy, subi_size=19, sigfactor=6, shift=False,
         See the documentation of the ``vip_hci.preproc.frame_shift`` function.
     fit_type: str, optional {'gaus','moff'}
         Type of 2d fit to infer the centroid of the satellite spots.
+    border_mode : {'reflect', 'nearest', 'constant', 'mirror', 'wrap'}
+        Points outside the boundaries of the input are filled accordingly.
+        With 'reflect', the input is extended by reflecting about the edge of
+        the last pixel. With 'nearest', the input is extended by replicating the
+        last pixel. With 'constant', the input is extended by filling all values
+        beyond the edge with zeros. With 'mirror', the input is extended by
+        reflecting about the center of the last pixel. With 'wrap', the input is
+        extended by wrapping around to the opposite edge. Default is 'reflect'.
     debug : bool, optional
         If True debug information is printed and plotted.
     verbose : bool, optional
@@ -376,7 +460,8 @@ def frame_center_satspots(array, xy, subi_size=19, sigfactor=6, shift=False,
 
             if shift:
                 array_rec = frame_shift(array, shifty, shiftx, imlib=imlib,
-                                        interpolation=interpolation)
+                                        interpolation=interpolation,
+                                        border_mode=border_mode)
                 return array_rec, shifty, shiftx, centy, centx
             else:
                 return shifty, shiftx
@@ -388,8 +473,8 @@ def frame_center_satspots(array, xy, subi_size=19, sigfactor=6, shift=False,
 
 
 def cube_recenter_satspots(array, xy, subi_size=19, sigfactor=6, plot=True,
-                           fit_type='moff', lbda=None, debug=False, verbose=True, 
-                           full_output=False):
+                           fit_type='moff', lbda=None, border_mode='constant', 
+                           debug=False, verbose=True, full_output=False):
     """ Function analog to frame_center_satspots but for image sequences. It
     actually will call frame_center_satspots for each image in the cube. The
     function also returns the shifted images (not recommended to use when the
@@ -406,7 +491,7 @@ def cube_recenter_satspots(array, xy, subi_size=19, sigfactor=6, plot=True,
     xy : tuple of 4 tuples of 2 elements
         Tuple with coordinates X,Y of the 4 satellite spots. When the spots are
         in an X configuration, the order is the following: top-left, top-right,
-        bottom-left and bottom-right. When the spots are in an + (cross-like)
+        bottom-left and bottom-right. When the spots are in an + (plus-like)
         configuration, the order is the following: top, right, left, bottom.
         If wavelength vector is not provided, assumes all sat spots of the cube
         are at a similar location. If wavelength is provided, only coordinates 
@@ -426,6 +511,14 @@ def cube_recenter_satspots(array, xy, subi_size=19, sigfactor=6, plot=True,
     lbda: 1d array or list, opt
         Wavelength vector. If provided, the subimages will be scaled accordingly 
         to follow the motion of the satellite spots.
+    border_mode : {'reflect', 'nearest', 'constant', 'mirror', 'wrap'}
+        Points outside the boundaries of the input are filled accordingly.
+        With 'reflect', the input is extended by reflecting about the edge of
+        the last pixel. With 'nearest', the input is extended by replicating the
+        last pixel. With 'constant', the input is extended by filling all values
+        beyond the edge with zeros. With 'mirror', the input is extended by
+        reflecting about the center of the last pixel. With 'wrap', the input is
+        extended by wrapping around to the opposite edge. Default is 'reflect'.
     debug : bool, optional
         If True debug information is printed and plotted (fit and residuals,
         intersections and shifts). This has to be used carefully as it can
@@ -477,7 +570,8 @@ def cube_recenter_satspots(array, xy, subi_size=19, sigfactor=6, plot=True,
     for i in Progressbar(range(n_frames), verbose=verbose):
         res = frame_center_satspots(array[i], final_xy[i], debug=debug, shift=True,
                                     subi_size=subi_size, sigfactor=sigfactor,
-                                    fit_type=fit_type, verbose=False)
+                                    fit_type=fit_type, verbose=False,
+                                    border_mode=border_mode)
         array_rec.append(res[0])
         shift_y[i] = res[1]
         shift_x[i] = res[2]
@@ -523,8 +617,7 @@ def cube_recenter_satspots(array, xy, subi_size=19, sigfactor=6, plot=True,
 
 def frame_center_radon(array, cropsize=None, hsize=0.4, step=0.01,
                        mask_center=None, nproc=None, satspots_cfg=None,
-                       full_output=False, verbose=True, 
-                       plot=True, debug=False):
+                       full_output=False, verbose=True, plot=True, debug=False):
     """ Finding the center of a broadband (co-added) frame with speckles and
     satellite spots elongated towards the star (center). We use the radon
     transform implementation from scikit-image.
@@ -582,7 +675,7 @@ def frame_center_radon(array, cropsize=None, hsize=0.4, step=0.01,
     if verbose:
         start_time = time_ini()
     frame = array.copy()
-    ori_cent, _ = frame_center(frame)
+    ori_cent_y, ori_cent_x = frame_center(frame)
     if cropsize is not None:
         if not cropsize%2:
             raise TypeError("If not None, cropsize should be odd integer")
@@ -655,7 +748,7 @@ def frame_center_radon(array, cropsize=None, hsize=0.4, step=0.01,
 
     cost_bound = costf.reshape(listyx.shape[0], listyx.shape[0])
     if plot:
-        plt.contour(cost_bound, cmap='CMRmap', origin='lower', lw=1, hold='on')
+        plt.contour(cost_bound, cmap='CMRmap', origin='lower')
         plt.imshow(cost_bound, cmap='CMRmap', origin='lower',
                    interpolation='nearest')
         plt.colorbar()
@@ -672,8 +765,8 @@ def frame_center_radon(array, cropsize=None, hsize=0.4, step=0.01,
     argmx = ind_maximax[int(np.ceil(num_max/2)) - 1]
     y_grid = np.array(coords)[:, 0].reshape(listyx.shape[0], listyx.shape[0])
     x_grid = np.array(coords)[:, 1].reshape(listyx.shape[0], listyx.shape[0])
-    optimy = y_grid[argmy, 0]+(ori_cent-cent)/2
-    optimx = x_grid[0, argmx]+(ori_cent-cent)/2
+    optimy = ori_cent_y+y_grid[argmy, 0]#+(ori_cent-cent)/2
+    optimx = ori_cent_x+x_grid[0, argmx]#+(ori_cent-cent)/2
 
     if verbose:
         print('Cost function max: {}'.format(costf.max()))
@@ -725,7 +818,8 @@ def _radon_costf(frame, cent, radint, coords, satspots_cfg=None):
 
 
 def cube_recenter_radon(array, full_output=False, verbose=True, imlib='vip-fft',
-                        interpolation='lanczos4', **kwargs):
+                        interpolation='lanczos4', border_mode='reflect', 
+                        **kwargs):
     """ Recenters a cube looping through its frames and calling the
     ``frame_center_radon`` function.
 
@@ -741,22 +835,18 @@ def cube_recenter_radon(array, full_output=False, verbose=True, imlib='vip-fft',
         See the documentation of the ``vip_hci.preproc.frame_shift`` function.
     interpolation : str, optional
         See the documentation of the ``vip_hci.preproc.frame_shift`` function.
-    cropsize : odd int, optional
-        Size in pixels of the cropped central area of the input array that will
-        be used. It should be large enough to contain the satellite spots.
-    hsize : float, optional
-        Size of the box for the grid search. The frame is shifted to each
-        direction from the center in a hsize length with a given step.
-    step : float, optional
-        The step of the coordinates change.
-    mask_center : None or int, optional
-        If None the central area of the frame is kept. If int a centered zero
-        mask will be applied to the frame. By default the center isn't masked.
-    nproc : int, optional
-        Number of processes for parallel computing. If None the number of
-        processes will be set to cpu_count()/2.
-    debug : bool, optional
-        Whether to print and plot intermediate info from ``frame_center_radon``.
+    border_mode : {'reflect', 'nearest', 'constant', 'mirror', 'wrap'}
+        Points outside the boundaries of the input are filled accordingly.
+        With 'reflect', the input is extended by reflecting about the edge of
+        the last pixel. With 'nearest', the input is extended by replicating the
+        last pixel. With 'constant', the input is extended by filling all values
+        beyond the edge with zeros. With 'mirror', the input is extended by
+        reflecting about the center of the last pixel. With 'wrap', the input is
+        extended by wrapping around to the opposite edge. Default is 'reflect'.
+    kwargs:
+        Additional optional parameters from vip_hci.preproc.frame_center_radon
+        function, such as cropsize, hsize, step, satspots_cfg, mask_center, 
+        nproc or debug.
 
 
     Returns
@@ -775,13 +865,15 @@ def cube_recenter_radon(array, full_output=False, verbose=True, imlib='vip-fft',
     n_frames = array.shape[0]
     x = np.zeros((n_frames))
     y = np.zeros((n_frames))
+    cy ,cx = frame_center(array[0])
     array_rec = array.copy()
 
     for i in Progressbar(range(n_frames), desc="frames", verbose=verbose):
         y[i], x[i] = frame_center_radon(array[i], verbose=False, plot=False,
                                         **kwargs)
-        array_rec[i] = frame_shift(array[i], y[i], x[i], imlib=imlib,
-                                   interpolation=interpolation)
+        array_rec[i] = frame_shift(array[i], cy-y[i], cx-x[i], imlib=imlib,
+                                   interpolation=interpolation, 
+                                   border_mode=border_mode)
 
     if verbose:
         timing(start_time)
@@ -795,9 +887,9 @@ def cube_recenter_radon(array, full_output=False, verbose=True, imlib='vip-fft',
 def cube_recenter_dft_upsampling(array, center_fr1=None, negative=False,
                                  fwhm=4, subi_size=None, upsample_factor=100,
                                  imlib='vip-fft', interpolation='lanczos4',
-                                 mask=None, full_output=False, verbose=True, 
-                                 nproc=1, save_shifts=False, debug=False, 
-                                 plot=True):
+                                 mask=None, border_mode='reflect', 
+                                 full_output=False, verbose=True, nproc=1, 
+                                 save_shifts=False, debug=False, plot=True):
     """ Recenters a cube of frames using the DFT upsampling method as
     proposed in Guizar et al. 2008 and implemented in the
     ``register_translation`` function from scikit-image.
@@ -836,7 +928,15 @@ def cube_recenter_dft_upsampling(array, center_fr1=None, negative=False,
     mask: 2D np.ndarray, optional
         Binary mask indicating where the cross-correlation should be calculated
         in the images. If provided, should be the same size as array frames.
-        [Note: only ysed uf version of skimage >= 0.18.0]
+        [Note: only used if version of skimage >= 0.18.0]
+    border_mode : {'reflect', 'nearest', 'constant', 'mirror', 'wrap'}
+        Points outside the boundaries of the input are filled accordingly.
+        With 'reflect', the input is extended by reflecting about the edge of
+        the last pixel. With 'nearest', the input is extended by replicating the
+        last pixel. With 'constant', the input is extended by filling all values
+        beyond the edge with zeros. With 'mirror', the input is extended by
+        reflecting about the center of the last pixel. With 'wrap', the input is
+        extended by wrapping around to the opposite edge. Default is 'reflect'.
     full_output : bool, optional
         Whether to return 2 1d arrays of shifts along with the recentered cube
         or not.
@@ -910,16 +1010,43 @@ def cube_recenter_dft_upsampling(array, center_fr1=None, negative=False,
     array_rec = array.copy()
 
     cy, cx = frame_center(array[0])
-    # Centroiding first frame with 2d gaussian and shifting
+
+    # Finding the shifts with DFT upsampling of each frame wrt the first
+    
+    if nproc == 1:
+        for i in Progressbar(range(1, n_frames), desc="frames", verbose=verbose):
+            y[i], x[i], array_rec[i] = _shift_dft(array_rec, array, i,
+                                                  upsample_factor, mask,
+                                                  interpolation, imlib,
+                                                  border_mode)
+    elif nproc > 1: 
+        res = pool_map(nproc, _shift_dft, array_rec, array, 
+                       iterable(range(1, n_frames)), upsample_factor, mask,
+                       interpolation, imlib, border_mode)
+        res = np.array(res)
+        
+        y[1:] = res[:,0]
+        x[1:] = res[:,1]
+        array_rec[1:] = [frames for frames in res[:,2]]
+        
+
+    if debug:
+        print("\nShifts in X and Y")
+        for i in range(n_frames):
+            print(x[i], y[i])
+            
+            
+    # Centroiding mean frame with 2d gaussian and shifting (only necessary if
+    # first frame was not well-centered)
     msg0 = "The rest of the frames will be shifted by cross-correlation wrt the" \
            " 1st"
     if subi_size is not None:
-        y1, x1 = _centroid_2dg_frame(array_rec, 0, subi_size, cy_1, cx_1,
-                                     negative, debug, fwhm)
-        x[0] = cx - x1
-        y[0] = cy - y1
-        array_rec[0] = frame_shift(array_rec[0], shift_y=y[0], shift_x=x[0],
-                                   imlib=imlib, interpolation=interpolation)
+        y1, x1 = _centroid_2dg_frame([np.mean(array_rec, axis=0)], 0, subi_size, 
+                                     cy_1, cx_1, negative, debug, fwhm)
+        x[:] += cx - x1
+        y[:] += cy - y1
+        array_rec = cube_shift(array, shift_y=y, shift_x=x, imlib=imlib, 
+                               interpolation=interpolation)
         if verbose:
             msg = "Shift for first frame X,Y=({:.3f}, {:.3f})"
             print(msg.format(x[0], y[0]))
@@ -937,28 +1064,6 @@ def cube_recenter_dft_upsampling(array, center_fr1=None, negative=False,
         x[0] = 0
         y[0] = 0
 
-    # Finding the shifts with DTF upsampling of each frame wrt the first
-    
-    if nproc == 1:
-        for i in Progressbar(range(1, n_frames), desc="frames", verbose=verbose):
-            y[i], x[i], array_rec[i] = _shift_dft(array_rec, array, i,
-                                                  upsample_factor, mask,
-                                                  interpolation, imlib)
-    elif nproc > 1: 
-        res = pool_map(nproc, _shift_dft, array_rec, array,
-                       iterable(range(1, n_frames)),
-                       upsample_factor, interpolation, imlib)
-        res = np.array(res)
-        
-        y[1:] = res[:,0]
-        x[1:] = res[:,1]
-        array_rec[1:] = [frames for frames in res[:,2]]
-        
-
-    if debug:
-        print("\nShifts in X and Y")
-        for i in range(n_frames):
-            print(x[i], y[i])
 
     if verbose:
         timing(start_time)
@@ -990,7 +1095,7 @@ def cube_recenter_dft_upsampling(array, center_fr1=None, negative=False,
 
 
 def _shift_dft(array_rec, array, frnum, upsample_factor, mask, interpolation, 
-               imlib):
+               imlib, border_mode):
     """
     function used in recenter_dft_unsampling
     """
@@ -1003,7 +1108,8 @@ def _shift_dft(array_rec, array, frnum, upsample_factor, mask, interpolation,
                              upsample_factor=upsample_factor)
     y_i, x_i = shift_yx
     array_rec_i = frame_shift(array[frnum], shift_y=y_i, shift_x=x_i,
-                              imlib=imlib, interpolation=interpolation)
+                              imlib=imlib, interpolation=interpolation,
+                              border_mode=border_mode)
     return y_i, x_i, array_rec_i    
 
 
@@ -1011,8 +1117,9 @@ def cube_recenter_2dfit(array, xy=None, fwhm=4, subi_size=5, model='gauss',
                         nproc=1, imlib='vip-fft', interpolation='lanczos4',
                         offset=None, negative=False, threshold=False, 
                         sigfactor=2, fix_neg=False, params_2g=None, 
-                        save_shifts=False, full_output=False, verbose=True, 
-                        debug=False, plot=True):
+                        border_mode='reflect', save_shifts=False, 
+                        full_output=False, verbose=True, debug=False, 
+                        plot=True):
     """ Recenters the frames of a cube. The shifts are found by fitting a 2d
     Gaussian or Moffat to a subimage centered at ``xy``. This assumes the frames
     don't have too large shifts (>5px). The frames are shifted using the
@@ -1073,6 +1180,14 @@ def cube_recenter_2dfit(array, xy=None, fwhm=4, subi_size=5, model='gauss',
     sigfactor: float, optional
         If thresholding is performed, set the the threshold in terms of 
         gaussian sigma in the subimage (will depend on your cropping size).
+    border_mode : {'reflect', 'nearest', 'constant', 'mirror', 'wrap'}
+        Points outside the boundaries of the input are filled accordingly.
+        With 'reflect', the input is extended by reflecting about the edge of
+        the last pixel. With 'nearest', the input is extended by replicating the
+        last pixel. With 'constant', the input is extended by filling all values
+        beyond the edge with zeros. With 'mirror', the input is extended by
+        reflecting about the center of the last pixel. With 'wrap', the input is
+        extended by wrapping around to the opposite edge. Default is 'reflect'.
     save_shifts : bool, optional
         Whether to save the shifts to a file in disk.
     full_output : bool, optional
@@ -1198,20 +1313,13 @@ def cube_recenter_2dfit(array, xy=None, fwhm=4, subi_size=5, model='gauss',
             print("\nShifts in X and Y")
             print(x[i], y[i])
         array_rec[i] = frame_shift(array[i], y[i], x[i], imlib=imlib,
-                                          interpolation=interpolation)
+                                          interpolation=interpolation,
+                                          border_mode=border_mode)
 
     if verbose:
         timing(start_time)
 
     if plot:
-        plt.figure(figsize=vip_figsize)
-        plt.plot(y, 'o-', label='shifts in y', alpha=0.5)
-        plt.plot(x, 'o-', label='shifts in x', alpha=0.5)
-        plt.legend(loc='best')
-        plt.grid('on', alpha=0.2)
-        plt.ylabel('Pixels')
-        plt.xlabel('Frame number')
-
         plt.figure(figsize=vip_figsize)
         b = int(np.sqrt(n_frames))
         la = 'Histogram'
@@ -1226,6 +1334,14 @@ def cube_recenter_2dfit(array, xy=None, fwhm=4, subi_size=5, model='gauss',
         plt.ylabel('Bin counts')
         plt.xlabel('Pixels')
 
+        plt.figure(figsize=vip_figsize)
+        plt.plot(y, 'o-', label='shifts in y', alpha=0.5)
+        plt.plot(x, 'o-', label='shifts in x', alpha=0.5)
+        plt.legend(loc='best')
+        plt.grid('on', alpha=0.2)
+        plt.ylabel('Pixels')
+        plt.xlabel('Frame number')
+
     if save_shifts:
         np.savetxt('recent_gauss_shifts.txt', np.transpose([y, x]), fmt='%f')
     if full_output:
@@ -1304,8 +1420,6 @@ def _centroid_2d2g_frame(cube, frnum, size, pos_y, pos_x, debug=False, fwhm=4,
     """
 
     size = min(cube[frnum].shape[0],cube[frnum].shape[1],size)
-    #sub_image, y1, x1 = get_square_robust(cube[frnum], size=size, y=pos_y, 
-    #                                      x=pos_x, position=True)
     if isinstance(params_2g,dict):
         fwhm_neg = params_2g.get('fwhm_neg', 0.8*fwhm)
         fwhm_pos = params_2g.get('fwhm_pos', 2*fwhm)
@@ -1343,8 +1457,8 @@ def cube_recenter_via_speckles(cube_sci, cube_ref=None, alignment_iter=5,
                                fwhm=4, debug=False, recenter_median=False,
                                fit_type='gaus', negative=True, crop=True,
                                subframesize=21, mask=None, imlib='vip-fft', 
-                               interpolation='lanczos4', plot=True, 
-                               full_output=False):
+                               interpolation='lanczos4', border_mode='reflect',
+                               plot=True, full_output=False):
     """ Registers frames based on the median speckle pattern. Optionally centers
     based on the position of the vortex null in the median frame. Images are
     filtered to isolate speckle spatial frequencies.
@@ -1390,6 +1504,14 @@ def cube_recenter_via_speckles(cube_sci, cube_ref=None, alignment_iter=5,
         Image processing library to use.
     interpolation : str, optional
         Interpolation method to use.
+    border_mode : {'reflect', 'nearest', 'constant', 'mirror', 'wrap'}
+        Points outside the boundaries of the input are filled accordingly.
+        With 'reflect', the input is extended by reflecting about the edge of
+        the last pixel. With 'nearest', the input is extended by replicating the
+        last pixel. With 'constant', the input is extended by filling all values
+        beyond the edge with zeros. With 'mirror', the input is extended by
+        reflecting about the center of the last pixel. With 'wrap', the input is
+        extended by wrapping around to the opposite edge. Default is 'reflect'.
     plot : bool, optional
         If True, the shifts are plotted.
     full_ouput: bool, optional
@@ -1519,7 +1641,8 @@ def cube_recenter_via_speckles(cube_sci, cube_ref=None, alignment_iter=5,
             
             alignment_cube[0] = frame_shift(alignment_cube[0, :, :], yshift,
                                             xshift, imlib=imlib,
-                                            interpolation=interpolation)
+                                            interpolation=interpolation,
+                                            border_mode=border_mode)
 
         # center the cube with stretched values
         cube_stret = np.log10((np.abs(alignment_cube) + 1) ** gammaval)
@@ -1539,7 +1662,8 @@ def cube_recenter_via_speckles(cube_sci, cube_ref=None, alignment_iter=5,
         for j in range(1, n_frames):
             alignment_cube[j] = frame_shift(alignment_cube[j], y_shift[j],
                                             x_shift[j], imlib=imlib,
-                                            interpolation=interpolation)
+                                            interpolation=interpolation,
+                                            border_mode=border_mode)
 
         cum_y_shifts += y_shift
         cum_x_shifts += x_shift
@@ -1550,7 +1674,8 @@ def cube_recenter_via_speckles(cube_sci, cube_ref=None, alignment_iter=5,
     for i in range(n):
         cube_reg_sci[i] = frame_shift(cube_sci[i], cum_y_shifts_sci[i],
                                       cum_x_shifts_sci[i], imlib=imlib,
-                                      interpolation=interpolation)
+                                      interpolation=interpolation,
+                                      border_mode=border_mode)
 
     if plot:
         plt.figure(figsize=vip_figsize)
@@ -1577,7 +1702,8 @@ def cube_recenter_via_speckles(cube_sci, cube_ref=None, alignment_iter=5,
         for i in range(nref):
             cube_reg_ref[i] = frame_shift(cube_ref[i], cum_y_shifts_ref[i],
                                           cum_x_shifts_ref[i], imlib=imlib,
-                                          interpolation=interpolation)
+                                          interpolation=interpolation,
+                                          border_mode=border_mode)
 
     if ref_star:
         if full_output:
