@@ -25,7 +25,7 @@ except ImportError:
 
 
 def sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3, 
-                 verbose=False):
+                 half_res_y=False, verbose=False):
     """Sigma filtering of pixels in a 2d array.
     
     Parameters
@@ -36,11 +36,18 @@ def sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3,
         Input array of the same size as frame_tmp, indicating the locations of
         bad/nan pixels by 1 (the rest of the array is set to 0)
     neighbor_box : int, optional
-        The side of the square window around each pixel where the sigma and
-        median are calculated.
+        The side of the window around each pixel where the sigma and
+        median are calculated. If half_res_y, this is the horizontal side (the
+        vertical side will be twice smaller).
     min_neighbors : int, optional
         Minimum number of good neighboring pixels to be able to correct the
         bad/nan pixels
+    half_res_y: bool, optional
+        Whether the input data have every couple of 2 rows identical, i.e. there
+        is twice less angular resolution vertically than horizontally (e.g.
+        SINFONI data). 
+    verbose: bool, optional
+        Whether to print more information while running.
         
     Returns
     -------
@@ -51,7 +58,8 @@ def sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3,
     if not no_numba:
         @njit
         def _sigma_filter_numba(frame_tmp, bpix_map, neighbor_box=3, 
-                                min_neighbors=3, verbose=False):
+                                min_neighbors=3, half_res_y=False, 
+                                verbose=False):
             if not frame_tmp.ndim == 2:
                 raise TypeError('Input array is not a frame or 2d array')
         
@@ -67,25 +75,29 @@ def sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3,
                 gp = 1 - bp                         # temporary good pixel map
                 for n in range(nb):
                     #0/ Determine the box around each pixel
-                    half_box = int(np.floor(neighbor_box/2.))
-                    hbox_b = min(half_box,wb[0][n])        # half size of the box at
+                    half_box_x = int(np.floor(neighbor_box/2.))
+                    if half_res_y:
+                        half_box_y = max(1,int(half_box_x/2))
+                    else:
+                        half_box_y = half_box_x
+                    hbox_b = min(half_box_y,wb[0][n])        # half size of the box at
                                                            # the bottom of the pixel
-                    hbox_t = min(half_box,sz_y-1-wb[0][n]) # half size of the box at 
+                    hbox_t = min(half_box_y,sz_y-1-wb[0][n]) # half size of the box at 
                                                            # the top of the pixel
-                    hbox_l = min(half_box,wb[1][n])        # half size of the box to 
+                    hbox_l = min(half_box_x,wb[1][n])        # half size of the box to 
                                                            # the left of the pixel
-                    hbox_r = min(half_box,sz_x-1-wb[1][n]) # half size of the box to 
+                    hbox_r = min(half_box_x,sz_x-1-wb[1][n]) # half size of the box to 
                                                            # the right of the pixel
                     # in case we are close to an edge, we want to extend the box
                     # in the direction opposite to the edge: 
                     if hbox_b<hbox_t: 
-                        hbox_t += half_box-hbox_b
+                        hbox_t += half_box_y-hbox_b
                     elif hbox_t<hbox_b: 
-                        hbox_b += half_box-hbox_t
+                        hbox_b += half_box_y-hbox_t
                     if hbox_l<hbox_r: 
-                        hbox_r += half_box-hbox_l
+                        hbox_r += half_box_x-hbox_l
                     elif hbox_r<hbox_l:
-                        hbox_l += half_box-hbox_r
+                        hbox_l += half_box_x-hbox_r
                     sgp = gp[(wb[0][n]-hbox_b):(wb[0][n]+hbox_t+1),
                              (wb[1][n]-hbox_l):(wb[1][n]+hbox_r+1)]
                     if int(np.sum(sgp)) >= min_neighbors:
@@ -107,8 +119,8 @@ def sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3,
 
     # TODO: If possible, replace this function using
     # scipy.ndimage.filters.generic_filter and astropy.stats.sigma_clip
-    def _sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3,
-                     verbose=False):
+    def _sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3, 
+                      half_res_y=False, verbose=False):
         """Same description as wrapper function."""
         
         if frame_tmp.ndim != 2:
@@ -127,19 +139,23 @@ def sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3,
             gp = 1 - bp                         # temporary good pixel map
             for n in range(nb):
                 #0/ Determine the box around each pixel
-                half_box = np.floor(neighbor_box/2)
-                hbox_b = min(half_box, wb[0][n])       # half size of the box at the
+                half_box_x = int(np.floor(neighbor_box/2.))
+                if half_res_y:
+                    half_box_y = max(1,int(half_box_x/2))
+                else:
+                    half_box_y = half_box_x
+                hbox_b = min(half_box_y, wb[0][n])       # half size of the box at the
                                                        # bottom of the pixel
-                hbox_t = min(half_box, sz_y-1-wb[0][n])# half size of the box at the
+                hbox_t = min(half_box_y, sz_y-1-wb[0][n])# half size of the box at the
                                                        # top of the pixel
-                hbox_l = min(half_box, wb[1][n])       # half size of the box to the
+                hbox_l = min(half_box_x, wb[1][n])       # half size of the box to the
                                                        # left of the pixel
-                hbox_r = min(half_box, sz_x-1-wb[1][n])# half size of the box to the
+                hbox_r = min(half_box_x, sz_x-1-wb[1][n])# half size of the box to the
                                                        # right of the pixel
-                # but in case we are at an edge, we want to extend the box by one 
-                # row/column of pixels in the direction opposite to the edge to 
-                # have 9 px instead of 6: 
-                if half_box == 1:
+                # but in case we are at an edge with min size box, we want to 
+                # extend the box by one row/column of pixels in the direction 
+                # opposite to the edge: 
+                if half_box_y == 1:
                     if wb[0][n] == sz_y-1:
                         hbox_b = hbox_b+1
                     elif wb[0][n] == 0:
@@ -172,7 +188,7 @@ def sigma_filter(frame_tmp, bpix_map, neighbor_box=3, min_neighbors=3,
 
 
 def clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
-               num_neighbor=3, mad=False):
+               num_neighbor=3, mad=False, half_res_y=False):
     """Sigma clipping for detecting outlying values in 2d array. If the
     parameter 'neighbor' is True the clipping can be performed in a local patch
     around each pixel, whose size depends on 'neighbor' parameter.
@@ -190,14 +206,19 @@ def clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
     neighbor : bool, optional
         For clipping over the median of the contiguous pixels.
     num_neighbor : int, optional
-        The side of the square window around each pixel where the sigma and 
-        median are calculated. 
+        The side of the window around each pixel where the sigma and
+        median are calculated. If half_res_y, this is the horizontal side (the
+        vertical side will be twice smaller).
     mad : {False, True}, bool optional
         If True, the median absolute deviation will be used instead of the 
         standard deviation.
     min_std : float, optional
         Min standard deviation considered for the lower and upper sigma 
         conditions. Can be useful for frames with padded edges (e.g. SPHERE/IFS)
+    half_res_y: bool, optional
+        Whether the input data have every couple of 2 rows identical, i.e. there
+        is twice less angular resolution vertically than horizontally (e.g.
+        SINFONI data). 
         
     Returns
     -------
@@ -211,7 +232,8 @@ def clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
     if not no_numba:
         @njit
         def _clip_array_numba(array, lower_sigma, upper_sigma, out_good=False, 
-                              neighbor=False, num_neighbor=3, mad=False): 
+                              neighbor=False, num_neighbor=3, mad=False,
+                              half_res_y=False): 
             if array.ndim != 2:
                 raise TypeError("Input array is not two dimensional (frame)\n")
                 
@@ -223,33 +245,41 @@ def clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
                 for y in range(ny):
                     for x in range(nx):
                         #0/ Determine the box around each pixel
-                        half_box = int(np.floor(num_neighbor/2.))
-                        hbox_b = min(half_box,y)    # half size of the box at the
+                    #0/ Determine the box around each pixel
+                        half_box_x = int(np.floor(num_neighbor/2.))
+                        if half_res_y:
+                            half_box_y = max(1,int(half_box_x/2))
+                        else:
+                            half_box_y = half_box_x
+                        hbox_b = min(half_box_y,y)    # half size of the box at the
                                                     # bottom of the pixel
-                        hbox_t = min(half_box,ny-y) # half size of the box at the
+                        hbox_t = min(half_box_y,ny-y) # half size of the box at the
                                                     # top of the pixel
-                        hbox_l = min(half_box,x)    # half size of the box to the
+                        hbox_l = min(half_box_x,x)    # half size of the box to the
                                                     # left of the pixel
-                        hbox_r = min(half_box,nx-x) # half size of the box to the
+                        hbox_r = min(half_box_x,nx-x) # half size of the box to the
                                                     # right of the pixel
                         # in case we are close to an edge, we want to extend the box
                         # in the direction opposite to the edge: 
                         if hbox_b<hbox_t: 
-                            hbox_t += half_box-hbox_b
+                            hbox_t += half_box_y-hbox_b
                         elif hbox_t<hbox_b: 
-                            hbox_b += half_box-hbox_t
+                            hbox_b += half_box_y-hbox_t
                         if hbox_l<hbox_r: 
-                            hbox_r += half_box-hbox_l
+                            hbox_r += half_box_x-hbox_l
                         elif hbox_r<hbox_l:
-                            hbox_l += half_box-hbox_r
+                            hbox_l += half_box_x-hbox_r
                             
                         sub_arr = array[y-hbox_b:y+hbox_t+1,
                                         x-hbox_l:x+hbox_r+1]
                         neighbours = sub_arr.flatten()
                         neigh_list = []
+                        remove_itself=True
                         for i in range(neighbours.shape[0]):
-                            neigh_list.append(neighbours[i])
-                        neigh_list.remove(array[y,x])
+                            if neighbours[i] == array[y,x] and remove_itself:
+                                remove_itself=False
+                            else:
+                                neigh_list.append(neighbours[i])
                         
                         neigh_arr = np.array(neigh_list)
                         median=np.median(neigh_arr)
@@ -282,8 +312,10 @@ def clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
                 return bad
             
     # TODO: If possible, replace this function using astropy.stats.sigma_clip
-    def _clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
-                    num_neighbor=3, mad=False):
+    def _clip_array(array, lower_sigma, upper_sigma, out_good=False, 
+                    neighbor=False, num_neighbor=3, mad=False, 
+                    half_res_y=False):
+        
         """Sigma clipping for detecting outlying values in 2d array. If the
         parameter 'neighbor' is True the clipping can be performed in a local patch
         around each pixel, whose size depends on 'neighbor' parameter.
@@ -301,11 +333,16 @@ def clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
         neighbor : bool optional
             For clipping over the median of the contiguous pixels.
         num_neighbor : int, optional
-            The side of the square window around each pixel where the sigma and 
-            median are calculated. 
+            The side of the window around each pixel where the sigma and
+            median are calculated. If half_res_y, this is the horizontal side (the
+            vertical side will be twice smaller).
         mad : bool, optional
             If True, the median absolute deviation will be used instead of the 
             standard deviation.
+        half_res_y: bool, optional
+            Whether the input data have every couple of 2 rows identical, i.e. there
+            is twice less angular resolution vertically than horizontally (e.g.
+            SINFONI data). 
             
         Returns
         -------
@@ -320,15 +357,21 @@ def clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
     
         values = array.copy()
         if neighbor and num_neighbor:
+            num_neighbor_x = num_neighbor
+            if half_res_y:
+                num_neighbor_y = max(int(num_neighbor/2),1)
+            else:
+                num_neighbor_y = num_neighbor
             median = generic_filter(array, function=np.median, 
-                                    size=(num_neighbor,num_neighbor), mode="mirror")
+                                    size=(num_neighbor_y,num_neighbor_x),
+                                    mode="mirror")
             if mad:
                 sigma = generic_filter(array, function=median_absolute_deviation,                                 
-                                       size=(num_neighbor,num_neighbor), 
+                                       size=(num_neighbor_y,num_neighbor_x), 
                                        mode="mirror")
             else:
                 sigma = generic_filter(array, function=np.std,                                 
-                                       size=(num_neighbor,num_neighbor), 
+                                       size=(num_neighbor_y,num_neighbor_x), 
                                        mode="mirror")
         else:
             median = np.median(values)
@@ -353,8 +396,9 @@ def clip_array(array, lower_sigma, upper_sigma, out_good=False, neighbor=False,
     if no_numba:
         return _clip_array(array, lower_sigma, upper_sigma, out_good=out_good, 
                            neighbor=neighbor, num_neighbor=num_neighbor, 
-                           mad=mad)
+                           mad=mad, half_res_y=half_res_y)
     else:
         return _clip_array_numba(array, lower_sigma, upper_sigma, 
                                  out_good=out_good, neighbor=neighbor, 
-                                 num_neighbor=num_neighbor, mad=mad)
\ No newline at end of file
+                                 num_neighbor=num_neighbor, mad=mad, 
+                                 half_res_y=half_res_y)
\ No newline at end of file
