@@ -14,13 +14,21 @@ __all__ = ['create_synth_psf',
 
 import numpy as np
 import pandas as pd
+from packaging import version
 import photutils
+if version.parse(photutils.__version__) >= version.parse('0.3'):
+    # for photutils version >= '0.3' use photutils.centroids.centroid_com
+    from photutils.centroids import centroid_com as cen_com
+else:
+    # for photutils version < '0.3' use photutils.centroid_com
+    import photutils.centroid_com as cen_com
 from hciplot import plot_frames
 from astropy.modeling import models, fitting
 from astropy.stats import (gaussian_sigma_to_fwhm, gaussian_fwhm_to_sigma,
                            sigma_clipped_stats)
-from .shapes import get_square, frame_center
-from ..conf import check_array
+from .coords import frame_center
+from .shapes import get_square
+from ..config import check_array
 
 
 def create_synth_psf(model='gauss', shape=(9, 9), amplitude=1, x_mean=None,
@@ -91,11 +99,11 @@ def create_synth_psf(model='gauss', shape=(9, 9), amplitude=1, x_mean=None,
         x, y = np.meshgrid(x, y)
 
         if model == 'gauss':
-            if isinstance(fwhm, (tuple, list)):
-                fwhm_x, fwhm_y = fwhm
-            else:
+            if np.isscalar(fwhm):
                 fwhm_y = fwhm
                 fwhm_x = fwhm
+            else:
+                fwhm_x, fwhm_y = fwhm
             gauss = models.Gaussian2D(amplitude=amplitude, x_mean=x_mean,
                                       y_mean=y_mean,
                                       x_stddev=fwhm_x * gaussian_fwhm_to_sigma,
@@ -118,7 +126,7 @@ def create_synth_psf(model='gauss', shape=(9, 9), amplitude=1, x_mean=None,
         return im
     # 3d case
     else:
-        if not isinstance(fwhm, (list, np.ndarray)):
+        if np.isscalar(fwhm):
             raise ValueError('`Fwhm` must be a 1d vector')
 
         cube = []
@@ -140,7 +148,7 @@ def fit_2dgaussian(array, crop=False, cent=None, cropsize=15, fwhmx=4, fwhmy=4,
     array : numpy ndarray
         Input frame with a single PSF.
     crop : bool, optional
-        If True an square sub image will be cropped.
+        If True a square sub image will be cropped equal to cropsize.
     cent : tuple of int, optional
         X,Y integer position of source in the array for extracting the subimage.
         If None the center of the frame is used for cropping the subframe (the
@@ -215,7 +223,7 @@ def fit_2dgaussian(array, crop=False, cent=None, cropsize=15, fwhmx=4, fwhmy=4,
 
     # Creating the 2D Gaussian model
     init_amplitude = np.ptp(psf_subimage)
-    xcom, ycom = photutils.centroid_com(psf_subimage)
+    xcom, ycom = cen_com(psf_subimage)
     gauss = models.Gaussian2D(amplitude=init_amplitude, theta=theta,
                               x_mean=xcom, y_mean=ycom,
                               x_stddev=fwhmx * gaussian_fwhm_to_sigma,
@@ -285,7 +293,7 @@ def fit_2dmoffat(array, crop=False, cent=None, cropsize=15, fwhm=4,
     array : numpy ndarray
         Input frame with a single PSF.
     crop : bool, optional
-        If True an square sub image will be cropped.
+        If True a square sub image will be cropped equal to cropsize.
     cent : tuple of int, optional
         X,Y integer position of source in the array for extracting the subimage.
         If None the center of the frame is used for cropping the subframe (the
@@ -349,7 +357,7 @@ def fit_2dmoffat(array, crop=False, cent=None, cropsize=15, fwhm=4,
 
     # Creating the 2D Moffat model
     init_amplitude = np.ptp(psf_subimage)
-    xcom, ycom = photutils.centroid_com(psf_subimage)
+    xcom, ycom = cen_com(psf_subimage)
     moffat = models.Moffat2D(amplitude=init_amplitude, x_0=xcom, y_0=ycom,
                              gamma=fwhm / 2., alpha=1)
     # Levenberg-Marquardt algorithm
@@ -416,7 +424,7 @@ def fit_2dairydisk(array, crop=False, cent=None, cropsize=15, fwhm=4,
     array : numpy ndarray
         Input frame with a single PSF.
     crop : bool, optional
-        If True an square sub image will be cropped.
+        If True a square sub image will be cropped equal to cropsize.
     cent : tuple of int, optional
         X,Y integer position of source in the array for extracting the subimage.
         If None the center of the frame is used for cropping the subframe (the
@@ -478,7 +486,7 @@ def fit_2dairydisk(array, crop=False, cent=None, cropsize=15, fwhm=4,
 
     # Creating the 2d Airy disk model
     init_amplitude = np.ptp(psf_subimage)
-    xcom, ycom = photutils.centroid_com(psf_subimage)
+    xcom, ycom = cen_com(psf_subimage)
     diam_1st_zero = (fwhm * 2.44) / 1.028
     airy = models.AiryDisk2D(amplitude=init_amplitude, x_0=xcom, y_0=ycom,
                              radius=diam_1st_zero/2.)
@@ -546,7 +554,7 @@ def fit_2d2gaussian(array, crop=False, cent=None, cropsize=15, fwhm_neg=4,
     array : numpy ndarray
         Input frame with a single PSF.
     crop : bool, optional
-        If True an square sub image will be cropped.
+        If True a square sub image will be cropped equal to cropsize.
     cent : tuple of float, optional
         X,Y position of the source in the array for extracting the 
         subimage. If None the center of the frame is used for cropping the 
@@ -651,7 +659,7 @@ def fit_2d2gaussian(array, crop=False, cent=None, cropsize=15, fwhm_neg=4,
 
     # Creating the 2D Gaussian model
     init_amplitude = np.ptp(psf_subimage)
-    #xcom, ycom = photutils.centroid_com(psf_subimage)
+    #xcom, ycom = cen_com(psf_subimage)
     ycom, xcom = frame_center(psf_subimage)
     fix_dico_pos = {'theta':True}
     bounds_dico_pos = {'amplitude':[0.8*init_amplitude,1.2*init_amplitude],
